const { execSync } = require('node:child_process')

/** the export is a higher-order function that takes
 * a gemini template as an arg. When the resulting template
 * is run, it first runs the gemini tempate and then the rest
 * of the code in this module, which transforms the gemini
 * output to a gophermap
 */
module.exports = geminiTemplate => (...args) => {
  const geminiText = geminiTemplate(...args)
  const geminiLines = geminiText.split(/\r?\n/)
  const gopherLines = geminiLines.flatMap(gemLine => {
    if (gemLine.indexOf('=>') == 0) {
      return gopherLink(gemLine)
    }
    else if (gemLine.trim().length) {
      // handle first chars that do something weird in a gophermap
      gemLine = gemLine.replace(/^#+/, '').replace(/^\*/, ' *')
      
      // in gemini, paragraphs have no line-ending character returns, so
      // that the client can soft wrap paragraph text as necessary. This
      // is not the gopher way, so so I'm using fmt command to hard-wrap
      // to 67 chars.
      //
      // This is done per line because `fmt` seems to mangle links
      const paragraph = execSync('fmt -w 67', { input: gemLine }).toString()
      const paragraphLines = paragraph.split('\n')
      return paragraphLines
    }
    return gemLine
  })

  gopherLines[0] = '!' + gopherLines[0].trim()

  return gopherLines.join('\n')
}

const geminiLinkRegex = /^=>\s+(\S+)(?:\s+(.*))?$/
const fileExtensionRegex = /(.*)(?:\.([^\/.]+))$/
const urlHasProtocolRegex = /^\w+\:\/\/.*$/


// convert gemini link to gopher.
// gopher uses a special first character for different kinds of links
// and the line must have a tab character in it.
const gopherLink = (gemLine) => {
  // line is format `=>\s+url\s+optional free text description
  const [_, url, alt] = gemLine.match(geminiLinkRegex) || []
  const [__, name, extension] = url?.match(fileExtensionRegex) || []
  const firstChar = gophermapItemType(extension, url)
  const gopherizedUrl = firstChar == 'h' ? 'URL:' + url : url
  gophermapLine = `${firstChar}${alt || ''}\t${gopherizedUrl}`
  return gophermapLine
}

const gophermapItemType = (extension, url) => {
  if (url.match(urlHasProtocolRegex)) {
    return 'h'
  }
  switch(extension) {
    case undefined:
      // it's a directory
      return '1'
    case 'jpg':
    case 'jpeg':
    case 'png':
      return 'I'
    case 'gif':
      return 'g'
  }
}

