const frontmatter = require('@github-docs/frontmatter')
const fs = require('node:fs')
const { execSync } = require('node:child_process')
const args = require('minimist-lite')(process.argv.slice(2))
const { DateTime } = require('luxon')

const geminiTemplate = require('./geminiTemplate.js')
const geminiIndexTemplate = require('./geminiIndexTemplate.js')
const geminiTagTemplate = require('./geminiTagTemplate.js')

const gopherize = require('./gopherize.js')
const gopherTemplate = gopherize(geminiTemplate)
const gopherIndexTemplate = gopherize(geminiIndexTemplate)
const gopherTagTemplate = gopherize(geminiTagTemplate)

const htmlTemplate = require('./htmlTemplate.js')
const htmlIndexTemplate = require('./htmlIndexTemplate.js')
const htmlTagTemplate = require('./htmlTagTemplate.js')
const rssFeed = require('./rssFeed.js')

const defaults = {
  root: '.',
  inputZip: null,
}

const { root, inputZip } = { ...defaults, ...args }

// Step 1. run the day-one-to-markdown script
inputZip && execSync(`~/day-one-to-markdown/day-one-to-markdown ${inputZip} ${root}`);

const generatedDirs = fs.readdirSync(root)

//execSync(`mogrify -auto-orient -resize 800x600 -strip '${root}/photos/*.jpeg[0]'`)

// for each folder in the dir
const processedFrontmatterData = generatedDirs.reduce((links, dir) => {
  const path = `${root}/${dir}`
  const relativeUrl = `${dir}`
  
  // check that it's a directory
  const pathInfo = fs.lstatSync(path)
  if (!pathInfo.isDirectory()) {
    console.log(`${path} is not a directory; ignoring`)
    return
  }

  // check that the directory contains index.markdown
  const indexFile = path + '/index.markdown'
  const indexInfo = fs.lstatSync(indexFile)
  if (!indexInfo.isFile()) {
    console.log(`${indexFile} doesn't exist or isn't a file; ignoring ${path}`)
    return
  }
  
  // clean up and resize the jpegs
  try {
    execSync(`mogrify -auto-orient -resize 800x800 -strip '${path}/*.jpeg[0]'`)
    execSync(`mogrify -auto-orient -resize 800x800 -strip '${path}/*.png[0]'`)
  } catch (e) {
  }

  // load the generated markdown, parse with frontmatter
  const thing = fs.readFileSync(path + '/index.markdown').toString()
  const document = frontmatter(thing)
  delete document.data.richText
  
  // Step 3. Generate the static content for rendering
  const markdown = document.content.trim()
  const dateTime = DateTime.fromJSDate(new Date(document.data.date), {zone: document.data.timeZone})
  const data = {
    // https://stackoverflow.com/a/75311261
    date: dateTime.toLocaleString(DateTime.DATE_SHORT, {locale: 'fr-CA'}),
    dateTime,
    location: document.data.location?.localityName || `TZ: ${document.data.timeZone}`,
    tags: document.data.tags.filter(it => it != 'public')
  }
  const firstLineBreak = markdown.indexOf('\n')
  const rawtitle = markdown.slice(0, firstLineBreak).replace(/^#+\s+/, '')
  const content = markdown.slice(firstLineBreak + 1).trim()

  // NOTE: parsing the title as gemini to unescape random markdown crap
  const title = execSync('md2gemini', { input: rawtitle }).toString().trim()
  const gemtext = execSync('md2gemini', { input: content }).toString().replace(/\%5C/g, '')
  const html = execSync('pandoc -f gfm', { input: content }).toString()

  console.log(title)
  const newLink = {path, date: data.date, relativeUrl, title, tags: data.tags, gemtext, html, data, dir}
  
  // add links to next/prev articles 
  if (links.length) {
    const prevLink = links[0]
    newLink.data.prev = `../${prevLink.relativeUrl}`
    prevLink.data.next = `../${newLink.relativeUrl}`
  }

  links.unshift(newLink)
  return links
}, [])

const tags = processedFrontmatterData.reduce((acc, entry) => {
  entry.tags.forEach(tag => (acc[tag] || (acc[tag] = [])).unshift(entry))
  return acc
}, {})

// now write the files out
execSync(`mkdir ${root}/tags`)
Object.entries(tags).forEach(([tag, entries]) => {
  execSync(`mkdir ${root}/tags/${tag}`)
  fs.writeFileSync(`${root}/tags/${tag}/index.gmi`, geminiTagTemplate(tag, entries)) 
  fs.writeFileSync(`${root}/tags/${tag}/gophermap`, gopherTagTemplate(tag, entries)) 
  fs.writeFileSync(`${root}/tags/${tag}/feed.xml`, rssFeed(entries, [tag], {
    title: `Feed for entries tagged ${tag}`,
    feed_url: `//tilde.club/~mycrobe/journal/tag/${tag}/feed.xml`,
  }))
  fs.writeFileSync(`${root}/tags/${tag}/index.html`, htmlTagTemplate(tag, entries)) 
})

processedFrontmatterData.forEach(({path, relativeUrl, title, gemtext, html, data}) => {
  const geminiPage = geminiTemplate(title, gemtext, data)
  const gopherPage = gopherTemplate(title, gemtext, data)
  const htmlPage = htmlTemplate(title, html, data)
  fs.writeFileSync(path + '/index.gmi', geminiPage)
  fs.writeFileSync(path + '/gophermap', gopherPage)
  fs.writeFileSync(path + '/index.html', htmlPage)
})

fs.writeFileSync(root + '/index.gmi', geminiIndexTemplate(processedFrontmatterData, Object.keys(tags)))
fs.writeFileSync(root + '/gophermap', gopherIndexTemplate(processedFrontmatterData, Object.keys(tags)))
fs.writeFileSync(root + '/feed.xml', rssFeed(processedFrontmatterData, Object.keys(tags)))
fs.writeFileSync(root + '/index.html', htmlIndexTemplate(processedFrontmatterData, Object.keys(tags)))

execSync(`cp ${__dirname}/assets/* ${root}`)
